import typer
from rich.console import Console

console = Console()
app = typer.Typer()


@app.command()
def not_implemented() -> None:
    console.print("Noting implemented yet")


typer_click_object = typer.main.get_command(app)

if __name__ == "__main__":
    app()



from fastapi import APIRouter, Request
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates
from pathlib import Path

router = APIRouter(
    prefix="/fe",
    responses={404: {"description": "Not Found!"}},
    include_in_schema=False,
)

templates = Jinja2Templates(
    directory=str(Path(__file__).resolve().parent.joinpath("templates"))
)


@router.get("/", response_class=HTMLResponse)
async def home(request: Request):
    context = {"title": "Home", "request": request}
    response = templates.TemplateResponse(
        "home.jinja2",
        context=context,
    )
    return response


@router.get("/search", response_class=HTMLResponse)
async def search(request: Request):
    context = {"title": "Search", "request": request}
    response = templates.TemplateResponse(
        "search.jinja2",
        context=context,
    )
    return response



from fastapi import FastAPI
from fastapi.responses import RedirectResponse, FileResponse
from fastapi.staticfiles import StaticFiles

from fact_explorer.app.db.db_session import db_instance as database_instance
from fact_explorer.app.entities.fact import FactOut
from fact_explorer.app.frontend.router import router as fe_router
from fact_explorer.config import get_configuration

from datetime import datetime, timedelta
from typing import Any, Dict, List
from json import loads, dumps
from cryptoshred.asynchronous.convenience import find_and_decrypt
from cryptoshred.backends import DynamoDbSsmBackend
from pathlib import Path

app = FastAPI()
app.mount(
    "/static",
    StaticFiles(
        directory=str(Path(__file__).resolve().parent.joinpath("frontend/static"))
    ),
    name="static",
)

app.include_router(fe_router)


@app.on_event("startup")
async def startup() -> None:
    await database_instance.connect()
    config = get_configuration()
    app.state.db = database_instance
    app.state.key_backend = DynamoDbSsmBackend(
        iv_param=config.cryptoshred_init_vector_path
    )


@app.get("/favicon.ico", include_in_schema=False)
async def favicon() -> FileResponse:
    return FileResponse(Path(__file__).parent.joinpath("frontend/static/favicon.png"))


@app.get("/", include_in_schema=False)
async def frontend_redirect() -> RedirectResponse:
    return RedirectResponse(url="/fe/")


@app.get("/health-check")
async def health_check() -> bool:
    return True


@app.get("/api/search", response_model=List[FactOut])
async def root(
    skip: int = 0,
    limit: int = 20,
    hq: str = "{}",
    pq: str = "{}",
    decrypt: bool = False,
) -> List[Dict[str, Any]]:
    """
    Query the Facts received. Results are ordered by when the where received.
    Newest first.

    - `skip`: skip the X newest entries
    - `limit`: display X facts
    - `hq`: the header query. Use query by example here
    Just send a dict of what you are looking for.
    - `pq`: the payload query. Use query by example here.
    Just send a dict of what you are looking for.
    - `decrypt`: whether or not to decrypt cryptoshred information
    (Caution might increase response time significantly)
    """
    response = await app.state.db.fetch_by_example(
        offset=skip, limit=limit, header_example=loads(hq), payload_example=loads(pq)
    )
    result = []
    if decrypt:
        for item in response:
            decrypted = await find_and_decrypt(
                x=loads(item["payload"]), key_backend=app.state.key_backend
            )
            result.append({"header": item["header"], "payload": dumps(decrypted)})

    else:
        result = response
    return result


@app.get("/api/last", response_model=List[FactOut])
async def last(minutes: int = 15, decrypt: bool = False) -> List[Dict[str, Any]]:
    """
    Displays the Facts received in the last X minutes. Results are ordered,
    newest first.

    Beware that this is a very expensive operations in terms of memory and time.
    Use with care, you will kill the application if you use to large values.

    - `minutes`: How many minutes from now backwards you want to get.
    - `decrypt`: whether or not to decrypt cryptoshred information
    (Caution might increase response time significantly)
    """
    minutes_ago = datetime.now() - timedelta(minutes=minutes)
    response = await app.state.db.fetch_by_time(until=minutes_ago)

    result = []
    if decrypt:
        for item in response:
            decrypted = await find_and_decrypt(
                x=[item["payload"]], key_backend=app.state.key_backend
            )
            result.append({"header": item["header"], "payload": decrypted[0]})

    else:
        result = response
    return result



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['fact_explorer',
 'fact_explorer.app',
 'fact_explorer.app.db',
 'fact_explorer.app.entities',
 'fact_explorer.app.frontend',
 'fact_explorer.cli']

package_data = \
{'': ['*'],
 'fact_explorer.app.frontend': ['static/*',
                                'templates/*',
                                'templates/fragments/*']}

install_requires = \
['aiofiles>=0.6.0,<0.7.0',
 'asyncpg>=0.22.0,<0.23.0',
 'cryptoshred>=0.0.6,<0.0.7',
 'fastapi>=0.65.0,<0.66.0',
 'psycopg2-binary>=2.8.6,<3.0.0',
 'rich>=10.4.0,<11.0.0',
 'typer>=0.3.2,<0.4.0',
 'uvicorn>=0.13.4,<0.14.0']

extras_require = \
{'docs': ['sphinx-rtd-theme>=0.5.2,<0.6.0',
          'sphinx-click>=3.0.1,<4.0.0',
          'Sphinx<4']}

entry_points = \
{'console_scripts': ['fact-explorer = fact_explorer.cli.main:app']}

setup_kwargs = {
    'name': 'fact-explorer',
    'version': '0.0.4',
    'description': 'A fast and simple tool to explore facts.',
    'long_description': "# Fact Explorer\n\nWelcome to fact_explorer. You can find more extensive documentation over at [readthedocs](https://fact-explorer.readthedocs.io/en/latest/).\n\nThis project arose manly out of the necessity to work with events directly instead of through projections. This is useful to:\n\n- Enable searching for 'random' events quickly during debugging\n- Checking events during migrations\n- Many other things (you should probably not do if you are an event purist)\n\nContributions are welcome. Just get in touch.\n\n## Quickstart\n\nSimply `pip install fact-exporter` and get going. The cli is available as `fact-exporter` and\nyou can run `fact-exporter --help` to get up to speed on what you can do.\n\n## Development\n\nThis project uses `poetry` for dependency management and `pre-commit` for local checks.\n",
    'author': 'Eduard Thamm',
    'author_email': 'eduard.thamm@thammit.at',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/edthamm/fact-explorer',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime
from functools import wraps
import asyncpg
from json import dumps
from typing import Any, Callable, Dict, Optional
from fact_explorer.app.entities.fact import FactOut
from fact_explorer.config import get_configuration


class Database:
    def __init__(self) -> None:
        self.config = get_configuration()
        self._cursor = None

        self._connection_pool = None
        self.con = None

    def connect_wrapper(func: Callable) -> Callable:  # type: ignore
        @wraps(func)
        async def wrapped(self, *args, **kwargs):  # noqa: ANN
            if not self._connection_pool:
                await self.connect()
            else:
                self.con = await self._connection_pool.acquire()
                try:
                    return await func(self, *args, **kwargs)
                except Exception as e:
                    print(e)
                finally:
                    await self._connection_pool.release(self.con)

        return wrapped

    async def connect(self) -> None:
        if not self._connection_pool:
            try:
                self._connection_pool = await asyncpg.create_pool(
                    min_size=1,
                    max_size=10,
                    command_timeout=60,
                    host=self.config.database_host,
                    port=self.config.database_port,
                    user=self.config.database_user,
                    password=self.config.database_password,
                    database=self.config.database_name,
                )

            except Exception as e:
                print(e)

    async def fetch_rows(self, query: str) -> Any:
        if not self._connection_pool:
            await self.connect()
        else:
            self.con = await self._connection_pool.acquire()
            try:
                result = await self.con.fetch(query)
                return result
            except Exception as e:
                print(e)
            finally:
                await self._connection_pool.release(self.con)

    @connect_wrapper
    async def fetch_by_example(
        self,
        *,
        header_example: Optional[Dict[str, str]] = None,
        payload_example: Optional[Dict[str, str]] = None,
        limit: int = 10,
        offset: int = 0
    ) -> Any:
        if not header_example:
            header_example = {}
        if not payload_example:
            payload_example = {}

        stmt = await self.con.prepare(  # type: ignore
            (
                "select *"
                " from fact"
                " where header @> cast($1 as jsonb)"
                " and payload @> cast ($2 as jsonb)"
                " order by ser desc"
                " limit $3::int"
                " offset $4::int"
            )
        )
        print(stmt.get_query())
        result = await stmt.fetch(
            dumps(header_example), dumps(payload_example), limit, offset
        )
        return result

    @connect_wrapper
    async def fetch_by_time(self, *, until: datetime) -> Any:
        """
        Will fetch the entries up to approximately until. Will fetch at least 50 rows
        iff available. Will fetch in batches afterwards. Since factcast does not have
        an index on _ts this is faster than a FTS although of course less than ideal
        """
        result = []
        async with self.con.transaction():  # type: ignore
            cursor = await self.con.cursor("select *" " from fact" " order by ser desc")  # type: ignore
            result += await cursor.fetch(50)
            last_retrieved_time = datetime.fromtimestamp(
                int(str(FactOut.parse_obj(result[-1]).header["meta"]["_ts"])[:10])  # type: ignore
            )
            while last_retrieved_time > until:
                # Doing this by fetching rows. it is way faster than a FTS.
                result += await cursor.fetch(50)
                last_retrieved_time = datetime.fromtimestamp(
                    int(str(FactOut.parse_obj(result[-1]).header["meta"]["_ts"])[:10])  # type: ignore
                )
            return result


db_instance = Database()
